@@ -1,5 +1,5 @@
 from maggma.api.resource import ReadOnlyResource
-from mp_api.routes.dielectric.models import DielectricDoc
+from emmet.core.polar import DielectricDoc
 
 from maggma.api.query_operator import PaginationQuery, SortQuery, SparseFieldsQuery
 from mp_api.routes.dielectric.query_operators import DielectricQuery
@@ -14,7 +14,7 @@ def dielectric_resource(dielectric_store):
             SortQuery(),
             PaginationQuery(),
             SparseFieldsQuery(
-                DielectricDoc, default_fields=["task_id", "last_updated"]
+                DielectricDoc, default_fields=["material_id", "last_updated"]
             ),
         ],
         tags=["Dielectric"],
