@@ -16,13 +16,14 @@ excluded_params = [
     "fields",
 ]
 
-sub_doc_fields = ["piezo"]  # type: list
+sub_doc_fields = []  # type: list
 
 alt_name_dict = {"piezoelectric_modulus": "e_ij_max"}  # type: dict
 
 custom_field_tests = {}  # type: dict
 
 
+@pytest.mark.xfail(reason="Needs deployment")
 @pytest.mark.skipif(
     os.environ.get("MP_API_KEY", None) is None, reason="No API key found."
 )
